#include <QApplication>
#include <QPlainTextEdit>
#include <QTextEdit>
#include <QMainWindow>
#include <QAction>
#include <QMenu>
#include <QMenuBar>
#include <QShortcut>
#include <QToolBar>
#include <QPrinter>
#include <QPrintPreviewDialog>

#include <QStyle>

#include <QDebug>
#include <QSysInfo>

#include <algorithm>
#include <iostream>

static const char text[] = R"(Headline 1

Paragraph 1: bla  bla  bla  bla  bla  bla  bla  bla  bla bla  bla bla  bla
bla  bla  bla  bla  bla  bla  bla  bla  bla bla  bla bla  bla

(Headline 2

Paragraph 2: bla  bla  bla  bla  bla  bla  bla  bla  bla bla  bla bla  bla
bla  bla  bla  bla  bla  bla  bla  bla  bla bla  bla bla  bla
)";

class EventFilter : public QObject {
public:

    EventFilter(QObject *p) : QObject(p) {}

    bool eventFilter(QObject *w, QEvent *e)
    {
        static int n = 0;
        switch (e->type()) {
        case QEvent::Resize:
        case QEvent::ShowToParent:
        case QEvent::HideToParent:
            qDebug() << '#' << n++ << w << e;
            break;
        default:
            break;
        }

        return QObject::eventFilter(w, e);
    }
};

static QString title()
{
    QString t = QLatin1String(QT_VERSION_STR);
#if QT_VERSION > 0x050000
    t += QLatin1Char(' ') + QGuiApplication::platformName();
#endif
    return t;
}

class PlainTextEdit : public QPlainTextEdit
{
    Q_OBJECT
public:
    using QPlainTextEdit::QPlainTextEdit;

public slots:
    void printPreview(QPrinter *p) { print(p); }
};

class MainWindow : public QMainWindow {
    Q_OBJECT
public:
    MainWindow();

public slots:
    void printPreview();

private:
    PlainTextEdit *m_pe;
};

MainWindow::MainWindow()
{
    QMenu *fileMenu = menuBar()->addMenu(tr("&File"));
    QAction *printPreviewAction = fileMenu->addAction(tr("&Print Preview"), this, SLOT(printPreview()));
    printPreviewAction->setShortcut(tr("Ctrl+P"));
    QAction *quitAction = fileMenu->addAction(tr("E&xit"), this, SLOT(close()));
    quitAction->setShortcut(tr("Ctrl+Q"));

    auto tb  = new QToolBar(this);
    addToolBar(Qt::TopToolBarArea, tb);
    tb->addAction(printPreviewAction);
    tb->addAction(quitAction);

    m_pe = new PlainTextEdit(text, this);
    setCentralWidget(m_pe);
}

void MainWindow::printPreview()
{
    QPrinter printer(QPrinter::HighResolution);    
    printer.setOrientation(QPrinter::Landscape);
    QPrintPreviewDialog preview(&printer, this);
    preview.setWindowTitle(windowTitle()+ " PrintPreview");
    connect(&preview, SIGNAL(paintRequested(QPrinter*)), m_pe, SLOT(printPreview(QPrinter*)));
    preview.exec();
}

int main(int argc, char *argv[])
{
    QStringList args;
    std::copy(argv + 1, argv + argc, std::back_inserter(args));

#if QT_VERSION > 0x050600
    QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
#endif
    QApplication app(argc, argv);
    if (!args.contains("-e"))
        app.installEventFilter(new EventFilter(&app));

    MainWindow w;
    QString t = title();
    qDebug() << t << args;
    w.setWindowTitle(t);
    w.show();
    return QCoreApplication::exec();
}

#include "main.moc"
